/**
 * @file
 *
 * @copyright Axiros GmbH 2013-2022, All rights reserved
 *
 * @brief Compiler specific handling
 */

#ifndef COMPILER_H_
#define COMPILER_H_

#ifndef __has_builtin
#define __has_builtin(x) 0
#endif

#ifndef __has_attribute
#define __has_attribute(x) 0
#endif

/************************************************
  Compiler specific flags used to remove warnings
 ************************************************/
#if defined(__GNUC__) || __has_attribute(unused)
	#define PARAMETER_IS_UNUSED __attribute__ ((unused))
#else
	#define PARAMETER_IS_UNUSED
#endif

/*************************************
  Compiler specific optimisation flags
 ************************************/
#if defined(__GNUC__) || __has_attribute(packed)
	#define PACKED __attribute__ ((packed))
#else
	#define PACKED
#endif

/************************************************
  Compiler specific annotations for code analysis
 ************************************************/
#if defined(__GNUC__) || __has_attribute(nonnull)
	#define ARGINDEX_NONNULL(index) __attribute__ ((nonnull(index)))
	#define ARGS_NONNULL __attribute__ ((nonnull))
#else
	#define ARGINDEX_NONNULL(index)
	#define ARGS_NONNULL
#endif

#if defined(__GNUC__) || __has_attribute(__format__)
	#define ARGS_FORMAT_PRINTF(fmt,args) __attribute__((__format__ (__printf__, fmt, args)))
#else
	#define ARGS_FORMAT_PRINTF(fmt,args)
#endif

#if defined(__GNUC__) || __has_attribute(malloc)
	#define MALLOC __attribute__ ((malloc))
#else
	#define MALLOC
#endif

#if __has_attribute(returns_nonnull)
	#define RETURNS_NONNULL __attribute__ ((returns_nonnull))
#else
	#define RETURNS_NONNULL
#endif

#if defined(__GNUC__) || __has_attribute(deprecated)
	#define DEPRECATED(why) __attribute__ ((deprecated(why)))
#else
	#define DEPRECATED(why)
#endif

#endif /* COMPILER_H_ */
