/**
 * @file
 *
 * @copyright Axiros GmbH 2015-2022, All rights reserved
 *
 * @brief Error code definitions
 */

#ifndef CORE_ERRORS_H_
#define CORE_ERRORS_H_

#include "compiler.h"

#define ANY_SUCCESS						0

#define USP_ERROR_MESSAGE_FAILED				7000
#define USP_ERROR_MESSAGE_NOT_SUPPORTED				7001
#define USP_ERROR_REQUEST_DENIED				7002
#define USP_ERROR_INTERNAL_ERROR				7003
#define USP_ERROR_INVALID_ARGUMENTS				7004
#define USP_ERROR_RESOURCES_EXCEEDED				7005
#define USP_ERROR_PERMISSION_DENIED				7006
#define USP_ERROR_INVALID_CONFIGURATION				7007
#define USP_ERROR_INVALID_PATH_SYNTAX				7008
#define USP_ERROR_PARAMETER_ACTION_FAILED			7009
#define USP_ERROR_UNSUPPORTED_PARAMETER				7010
#define USP_ERROR_INVALID_TYPE					7011
#define USP_ERROR_INVALID_VALUE					7012
#define USP_ERROR_ATTEMPT_TO_UPDATE_NON_WRITEABLE_PARAMETER	7013
#define USP_ERROR_VALUE_CONFLICT				7014
#define USP_ERROR_OPERATION_ERROR				7015
#define USP_ERROR_OBJECT_DOES_NOT_EXIST				7016
#define USP_ERROR_OBJECT_COULD_NOT_BE_CREATED			7017
#define USP_ERROR_OBJECT_IS_NOT_A_TABLE				7018
#define USP_ERROR_ATTEMPT_TO_CREATE_NON_CREATABLE_OBJECT	7019
#define USP_ERROR_OBJECT_COULD_NOT_BE_UPDATED			7020
#define USP_ERROR_REQUIRED_PARAMETER_FAILED			7021
#define USP_ERROR_COMMAND_FAILURE				7022
#define USP_ERROR_COMMAND_CANCELED				7023
#define USP_ERROR_DELETE_FAILURE				7024
#define USP_ERROR_OBJECT_EXISTS_WITH_DUPLICATE_KEY		7025
#define USP_ERROR_INVALID_PATH					7026
#define USP_ERROR_INVALID_COMMAND_ARGUMENTS			7027


/* NOTE: This needs to line up with the USP specification */
enum usp_result
{
	USP_SUCCESS = ANY_SUCCESS,
	USP_MESSAGE_FAILED = USP_ERROR_MESSAGE_FAILED,
	USP_MESSAGE_NOT_SUPPORTED = USP_ERROR_MESSAGE_NOT_SUPPORTED,
	USP_REQUEST_DENIED = USP_ERROR_REQUEST_DENIED,
	USP_INTERNAL_ERROR = USP_ERROR_INTERNAL_ERROR,
	USP_INVALID_ARGUMENTS = USP_ERROR_INVALID_ARGUMENTS,
	USP_RESOURCES_EXCEEDED = USP_ERROR_RESOURCES_EXCEEDED,
	USP_PERMISSION_DENIED = USP_ERROR_PERMISSION_DENIED,
	USP_INVALID_CONFIGURATION = USP_ERROR_INVALID_CONFIGURATION,
	USP_INVALID_PATH_SYNTAX = USP_ERROR_INVALID_PATH_SYNTAX,
	USP_PARAMETER_ACTION_FAILED = USP_ERROR_PARAMETER_ACTION_FAILED,
	USP_UNSUPPORTED_PARAMETER = USP_ERROR_UNSUPPORTED_PARAMETER,
	USP_INVALID_TYPE = USP_ERROR_INVALID_TYPE,
	USP_INVALID_VALUE = USP_ERROR_INVALID_VALUE,
	USP_ATTEMPT_TO_UPDATE_NON_WRITEABLE_PARAMETER = USP_ERROR_ATTEMPT_TO_UPDATE_NON_WRITEABLE_PARAMETER,
	USP_VALUE_CONFLICT = USP_ERROR_VALUE_CONFLICT,
	USP_OPERATION_ERROR = USP_ERROR_OPERATION_ERROR,
	USP_OBJECT_DOES_NOT_EXIST = USP_ERROR_OBJECT_DOES_NOT_EXIST,
	USP_OBJECT_COULD_NOT_BE_CREATED = USP_ERROR_OBJECT_COULD_NOT_BE_CREATED,
	USP_OBJECT_IS_NOT_A_TABLE = USP_ERROR_OBJECT_IS_NOT_A_TABLE,
	USP_ATTEMPT_TO_CREATE_NON_CREATABLE_OBJECT = USP_ERROR_ATTEMPT_TO_CREATE_NON_CREATABLE_OBJECT,
	USP_OBJECT_COULD_NOT_BE_UPDATED = USP_ERROR_OBJECT_COULD_NOT_BE_UPDATED,
	USP_REQUIRED_PARAMETER_FAILED = USP_ERROR_REQUIRED_PARAMETER_FAILED,
	USP_COMMAND_FAILURE = USP_ERROR_COMMAND_FAILURE,
	USP_COMMAND_CANCELED = USP_ERROR_COMMAND_CANCELED,
	USP_DELETE_FAILURE = USP_ERROR_DELETE_FAILURE,
	USP_OBJECT_EXISTS_WITH_DUPLICATE_KEY = USP_ERROR_OBJECT_EXISTS_WITH_DUPLICATE_KEY,
	USP_INVALID_PATH = USP_ERROR_INVALID_PATH,
	USP_INVALID_COMMAND_ARGUMENTS = USP_ERROR_INVALID_COMMAND_ARGUMENTS,
};

#define CWMP_ERROR_9000_METHOD_NOT_SUPPORTED					9000
#define CWMP_ERROR_9001_REQUEST_DENIED						9001
#define CWMP_ERROR_9002_INTERNAL_ERROR						9002
#define CWMP_ERROR_9003_INVALID_ARGUMENTS					9003
#define CWMP_ERROR_9004_RESOURCES_EXCEEDED					9004
#define CWMP_ERROR_9005_INVALID_PARAMETER_NAME					9005
#define CWMP_ERROR_9006_INVALID_PARAMETER_TYPE					9006
#define CWMP_ERROR_9007_INVALID_PARAMETER_VALUE					9007
#define CWMP_ERROR_9008_ATTEMPT_TO_SET_NON_WRITABLE_PARAMETER			9008
#define CWMP_ERROR_9009_NOTIFICATION_REQUEST_REJECTED				9009
#define CWMP_ERROR_9010_FILE_TRANSFER_FAILURE					9010
#define CWMP_ERROR_9011_UPLOAD_FAILURE						9011
#define CWMP_ERROR_9012_FILE_TRANSFER_SERVER_AUTH_FAILURE			9012
#define CWMP_ERROR_9013_UNSUPPORTED_PROTOCOL					9013
#define CWMP_ERROR_9014_UNABLE_TO_JOIN_MULTICAST_GROUP				9014
#define CWMP_ERROR_9015_UNABLE_TO_CONTACT_FILE_SERVER				9015
#define CWMP_ERROR_9016_UNABLE_TO_ACCESS_FILE					9016
#define CWMP_ERROR_9017_UNABLE_TO_COMPLETE_DOWNLOAD				9017
#define CWMP_ERROR_9018_FILE_CORRUPTED_OR_OTHERWISE_UNUSABLE			9018
#define CWMP_ERROR_9019_FILE_AUTHENTICATION_FAILURE				9019
#define CWMP_ERROR_9020_UNABLE_TO_COMPLETE_DOWNLOAD_WITHIN_SPECIFIED_TIME	9020
#define CWMP_ERROR_9021_CANCEL_OF_TRANSFER_NOT_PERMITTED_IN_CURR_STATE		9021

enum cwmp_result {
	CWMP_SUCCESS = ANY_SUCCESS,
	CWMP_METHOD_NOT_SUPPORTED = CWMP_ERROR_9000_METHOD_NOT_SUPPORTED,
	CWMP_REQUEST_DENIED = CWMP_ERROR_9001_REQUEST_DENIED,
	CWMP_INTERNAL_ERROR = CWMP_ERROR_9002_INTERNAL_ERROR,
	CWMP_INVALID_ARGUMENTS = CWMP_ERROR_9003_INVALID_ARGUMENTS,
	CWMP_RESOURCES_EXCEEDED = CWMP_ERROR_9004_RESOURCES_EXCEEDED,
	CWMP_INVALID_PARAMETER_NAME = CWMP_ERROR_9005_INVALID_PARAMETER_NAME,
	CWMP_INVALID_PARAMETER_TYPE = CWMP_ERROR_9006_INVALID_PARAMETER_TYPE,
	CWMP_INVALID_PARAMETER_VALUE = CWMP_ERROR_9007_INVALID_PARAMETER_VALUE,
	CWMP_ATTEMPT_TO_SET_NON_WRITABLE_PARAMETER = CWMP_ERROR_9008_ATTEMPT_TO_SET_NON_WRITABLE_PARAMETER,
	CWMP_NOTIFICATION_REQUEST_REJECTED = CWMP_ERROR_9009_NOTIFICATION_REQUEST_REJECTED,
	CWMP_FILE_TRANSFER_FAILURE = CWMP_ERROR_9010_FILE_TRANSFER_FAILURE,
	CWMP_UPLOAD_FAILURE = CWMP_ERROR_9011_UPLOAD_FAILURE,
	CWMP_FILE_TRANSFER_SERVER_AUTH_FAILURE = CWMP_ERROR_9012_FILE_TRANSFER_SERVER_AUTH_FAILURE,
	CWMP_UNSUPPORTED_PROTOCOL = CWMP_ERROR_9013_UNSUPPORTED_PROTOCOL,
	CWMP_UNABLE_TO_JOIN_MULTICAST_GROUP = CWMP_ERROR_9014_UNABLE_TO_JOIN_MULTICAST_GROUP,
	CWMP_UNABLE_TO_CONTACT_FILE_SERVER = CWMP_ERROR_9015_UNABLE_TO_CONTACT_FILE_SERVER,
	CWMP_UNABLE_TO_ACCESS_FILE = CWMP_ERROR_9016_UNABLE_TO_ACCESS_FILE,
	CWMP_UNABLE_TO_COMPLETE_DOWNLOAD = CWMP_ERROR_9017_UNABLE_TO_COMPLETE_DOWNLOAD,
	CWMP_FILE_CORRUPTED_OR_OTHERWISE_UNUSABLE = CWMP_ERROR_9018_FILE_CORRUPTED_OR_OTHERWISE_UNUSABLE,
	CWMP_FILE_AUTHENTICATION_FAILURE = CWMP_ERROR_9019_FILE_AUTHENTICATION_FAILURE,
	CWMP_UNABLE_TO_COMPLETE_DOWNLOAD_WITHIN_SPECIFIED_TIME = CWMP_ERROR_9020_UNABLE_TO_COMPLETE_DOWNLOAD_WITHIN_SPECIFIED_TIME,
	CWMP_CANCEL_OF_TRANSFER_NOT_PERMITTED_IN_CURR_STATE = CWMP_ERROR_9021_CANCEL_OF_TRANSFER_NOT_PERMITTED_IN_CURR_STATE,
};


/**
 * @brief Returns a human readable version of a TR-069 or USP result
 *
 * @param[in] errval	the error number to resolve
 *
 * @returns A const pointer to human readable string matching the error or NULL if the error is not known
 */
const char *errors_to_str (unsigned int errval);


/**
 * @brief Returns a human readable version of a TR-069 or USP result, falling back to a default if error is not known
 *
 * @param[in] errval	the error number to resolve
 *
 * @returns A const pointer to a non-NULL human readable string matching the error
 */
const char *errors_to_str_nofail (unsigned int errval) RETURNS_NONNULL;

#endif /* CORE_ERRORS_H_ */
