/**
 * @file Data model definitions
 *
 *  @copyright Axiros GmbH 2014-2022, All rights reserved
 *
 * @brief Data model definitions
 */


#ifndef DATAMODEL_H_
#define DATAMODEL_H_

/* Forward declarations */
struct dm_object;
struct dm_parameter;

#define ALIASLENGTH 64     /* TR-106 */
#define MAXPATHLENGTH 256  /* TR-106 */
#define ZEROTIME "0001-01-01T00:00:00Z"

/**
 * @brief return codes used by dm functions
 */
typedef enum {
	DM_RC_OK = 0,
	DM_RC_ERROR,
	DM_RC_ALLOCATION_ERROR,
	DM_RC_FILE_NOT_FOUND,
	DM_RC_ARGS_NOT_FOUND,
	DM_RC_FILE_CORRUPTION
} dm_returncode;


/**
 * @brief Internal type representation
 */
typedef enum {
	dm_xsd_string = 0,
	dm_xsd_int,
	dm_xsd_long,
	dm_xsd_bool,
	dm_xsd_unsignedInt,
	dm_xsd_unsignedLong,
	dm_xsd_datetime,
	dm_xsd_base64,
	dm_xsd_hexBinary,
	dm_xsd_decimal
} dm_xsd_type;


/**
 * @brief Internal type representation for syntax facets
 */
typedef enum {
	dm_syn_none = 0,
	dm_syn_enumeration,
	dm_syn_pattern,
	dm_syn_ipv4address,
	dm_syn_ipv6address,
	dm_syn_ipaddress,
	dm_syn_macaddress,
	dm_syn_alias,
	dm_syn_stringsize,
	dm_syn_intrange,
	dm_syn_uintrange,
	dm_syn_longrange,
	dm_syn_ulongrange,
	dm_syn_numentry,
	dm_syn_enumref,
	dm_syn_custom
} dm_syntax_facet;


typedef struct {
	unsigned int	min_size;
	unsigned int	max_size;
} dm_syntax_stringsize;


typedef struct {
	int	min;
	int	max;
} dm_syntax_intrange;


typedef struct {
	unsigned int	min;
	unsigned int	max;
} dm_syntax_uintrange;


typedef struct {
	long	min;
	long	max;
} dm_syntax_longrange;


typedef struct {
	unsigned long	min;
	unsigned long	max;
} dm_syntax_ulongrange;


typedef struct {
	char *ref;
} dm_syntax_enumref;


typedef int (*dm_readfunc)  (struct dm_object *obj);
typedef int (*dm_writefunc) (struct dm_object *obj, struct dm_parameter* par, char* value, int check);

#endif /* DATAMODEL_H_ */
