/**
 * @file
 *
 * @copyright Axiros GmbH 2014-2022, All rights reserved
 *
 * @brief Data model persistency
 */

#ifndef OSADAPTER_DATAMODEL_H
#define OSADAPTER_DATAMODEL_H

#include "compiler.h"

#include "datamodel/DataModel.h"


/* Forward declarations */
struct dm_object;


/**
 * @brief save the complete data model to storage
 */
void os_dm_save_all (struct dm_object *root, const char *filename) ARGS_NONNULL;


/**
 * @brief read the data model from storage
 *
 * @param[in] root root object of the datamodel to fill
 * @param[in] filename file to read data from
 *
 * @return  is either 0 for success or any other value for error
 */
dm_returncode os_dm_read_all (struct dm_object *root, const char* filename) ARGS_NONNULL;


/**
 * @brief setup the whole base data model(s) to use
 *
 * @return a pointer to the root object of the datamodel
 *
 * @details this function is supposed to mount the root data model and
 * potentially also additional service data models and configure them for later
 * use
 */
struct dm_object *os_dm_setup (void);


/**
 * @brief initialise the data model with factory defaults
 *
 * @param[in] dm_root	root object of the datamodel to fill
 *
 * @return  is either 0 for success or any other value for error
 */
dm_returncode os_dm_init_defaults (struct dm_object *dm_root) ARGS_NONNULL;

#endif /* OSADAPTER_DATAMODEL_H */
