/**
 * @file
 *
 * @copyright Axiros GmbH 2010-2022, All rights reserved
 *
 * @brief Generic networking functionality outside of TR-069 HTTP(S) transport
 *
 */


#include <stdint.h>


#ifndef OSADAPTER_NETWORK_H_
#define OSADAPTER_NETWORK_H_

#include "compiler.h"

struct frontend;

typedef void (*network_probe_cb) (struct frontend *fe, char *ip);

/**
 * @brief Probe for outgoing IP directed to ACS return a statically allocated
 *        string with the current one
 *
 * @param[in] addr	host name to open the TCP connection to
 * @param[in] port	port to open the TCP connection to
 * @param[in] use_ipv6	1 if we want to use IPv6 (if available) or 0 for just IPv4
 * @param[in] fe	the frontend to pass along when calling the callback
 * @param[in] callback	the callback to call after retrieving the outgoing ip
 */
void network_probe_outgoing_ip(const char * addr, uint16_t port, int use_ipv6, struct frontend *fe, network_probe_cb callback) ARGS_NONNULL;

#endif /* OSADAPTER_NETWORK_H_ */
